"""
 Coloring Book nitty gritty..

 The main internals of a coloring book. Each book consists of a color
 selection box and a canvas. In addition are buttons for paging through
 the images in the book, a button for taking a snapshot of the canvas, and
 a box that shows the currently selected color.

 Each canvas is a page from the coloring book composed of image layers. Each
 layer is an area of the coloring book that can be colored.
"""

import images, lang, sounds, sys, tempfile, text, ui
from locals import *

class book:
	def __init__ ( self, color = None ):
		self.changed = True
		self.messages = pygame.sprite.Group ()
		self.groups = [ pygame.sprite.RenderPlain (), pygame.sprite.RenderPlain (), pygame.sprite.RenderPlain (), pygame.sprite.RenderPlain () ]
		if not color:
			color = DEFAULT_COLOR
		self.Selected_color = selected_color ( self, self.groups[1], [ 124, 434 ], color )
		self.Left_button = left_button ( self, self.groups[1], [ 5, 230 ] )
		self.Right_button = right_button ( self, self.groups[1], [ 624, 230 ] )
		self.Tool_button = tool_button ( self, self.groups[1], [ 80, 436 ] )
		self.Undo_button = undo_button ( self, self.groups[1], [ 500, 436 ] )
		self.Snapshot_button = snapshot_button ( self, self.groups[1], [ 550, 438 ] )
		self.Colors = colors ( self, self.groups[1], [ 169, 434 ] )
		self.Cursor = cursor ( self, self.groups[3], pygame.mouse.get_pos () )
		self.Chrome = chrome ( self, self.groups[0] )
		self.Pad = pad ( self, self.groups[2], [ 19, 19 ] )
		self.doodle = False
		self.start_canvas ()

	def list_pages ( self ):
		if not BOOK_PAGES:
			sys.exit ( lang.invalid_book )
		return BOOK_PAGES

	def start_page ( self ):
		self.page = 0
		return self.list_pages ()[0]

	def prev_page ( self ):
		if self.page == 0:
			self.page = len ( self.list_pages () ) - 1
		else:
			self.page = self.page - 1
		return self.list_pages ()[self.page]

	def next_page ( self ):	
		if self.page == len ( self.list_pages () ) - 1:
			self.page = 0
		else:
			self.page = self.page + 1
		return self.list_pages ()[self.page]

	def start_canvas ( self ):
		self.Canvas = canvas ( self, self.groups[1], [ 19, 19 ], self.start_page () )
		self.init_history ()
		self.Canvas.draw ()

	def prev_canvas ( self ):
		self.Canvas.kill ()
		self.Pad.reset ()
		self.Canvas = canvas ( self, self.groups[1], [ 19, 19 ], self.prev_page () )
		self.init_history ()
		self.Canvas.draw ()

	def next_canvas ( self ):
		self.Canvas.kill ()
		self.Pad.reset ()
		self.Canvas = canvas ( self, self.groups[1], [ 19, 19 ], self.next_page () )
		self.init_history ()
		self.Canvas.draw ()

	def check_cursor ( self, pos, buttons ):
		for group in self.groups:
			for sprite in group.sprites ():
				if sprite.check_cursor ( pos, buttons ):
					self.changed = True

	def check_click ( self, pos ):
		for group in self.groups:
			for sprite in group.sprites ():
				if sprite.check_click ( pos ):
					self.changed = True
					return
	def init_history ( self ):
		self.history = []
		for layer in self.Canvas.layers:
			self.history.append ( [ layer, layer.color ] )

	def draw ( self ):
		for sprite in self.messages.sprites ():
			self.changed = True
			sprite.shift ()
		if self.changed:
			self.changed = False
			return self.groups
		return None

	def tool ( self ):
		if self.doodle:
			self.doodle = False
		else:
			self.doodle = True
		self.Cursor.change_cursor ()

	def paint_layer ( self, layer ):
		self.history.append ( [ layer, layer.color ] )
		layer.set_color ( self.Selected_color.color )
		self.Canvas.draw ()

	def set_color ( self, color ):
		if color is CANVAS_MASK:
			color[2] = color[2] - 1
		self.Selected_color.set_color ( color )

	def undo ( self ):
		try:
			last = self.history.pop ()
			last[0].set_color ( last[1] )
			self.Canvas.draw ()
		except IndexError:
			pass

	def snapshot ( self ):
		canvas = self.Canvas.draw ()
		canvas.blit ( self.Pad.image, ( 0, 0 ) )
		fd, filename = tempfile.mkstemp ( '.tga', 'kavlon-', SNAPSHOT_DIR )
		pygame.image.save ( canvas, filename )
		self.messages.add ( message ( self, self.groups[3], lang.snapshot % ( filename ) ) )

class layer:
	def __init__ ( self, surface, rect, offset, color = None, name = None ):
		self.surface = surface
		self.rect = rect
		self.offset = offset
		self.size = ( rect[2], rect[3] )
		self.background = pygame.Surface ( self.size ).convert ()
		if not color:
			color = BACKGROUND_COLOR
		self.set_color ( color )
		if not name:
			name = ''
		self.name = name

	def check_click ( self, pos ):
		if pos[0] >= self.offset[0] and pos[1] >= self.offset[1] and pos[0] <= ( self.offset[0] + self.size[0] ) and pos[1] <= ( self.offset[1] + self.size[1] ):
			p = ( pos[0] - self.offset[0] + self.rect[0], pos[1] - self.offset[1] + self.rect[1] )
			try:
				if self.surface.get_at ( p ) != CANVAS_MASK:
					return True
			except IndexError:
				pass
		return False

	def set_color ( self, color ):
		self.color = color
		self.background.fill ( color )
		self.surface.set_colorkey ( LAYER_MASK )
		self.background.blit ( self.surface, ( 0, 0 ), self.rect )

	def draw ( self, surface ):
		self.background.set_colorkey ( CANVAS_MASK )
		surface.blit ( self.background, self.offset )

class canvas ( ui.Sprite ):
	def __init__ ( self, cb, group, pos, page ):
		ui.Sprite.__init__ ( self, cb, group )
		self.image = pygame.Surface ( CANVAS_SIZE ).convert ()
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.page = page
		self.background ()
		self.load_layers ()
		self.sound = sounds.load_sound ( PAINTING_SOUND )

	def draw ( self ):
		for layer in self.layers:
			layer.draw ( self.image )
		return self.image

	def background ( self ):
		background = pygame.Surface ( CANVAS_SIZE ).convert ()
		background.fill ( LAYER_MASK )
		self.layers = [ layer ( background, ( 0, 0, 600, 400 ), ( 0, 0 ), name = 'background' ) ]

	def load_layers ( self ):
		page = {}
		execfile ( self.page, page )
		if page['image'] and page['map']:
			img = images.load_image ( page['image'] )
			for m in page['map']:
				self.layers.append ( layer ( img, m[0], m[1] ) )

	def check_click ( self, pos ):
		if ( not self.cb.doodle ) and self.rect.collidepoint ( pos[0], pos[1] ):
			pos = [ pos[0] - self.rect.left, pos[1] - self.rect.top ]
			for layer in self.layers[::-1]:
				if layer.check_click ( pos ):
					sounds.play_sound ( self.sound )
					self.cb.paint_layer ( layer )
					return True
		return False

class colors ( ui.Sprite ):
	def __init__ ( self, cb, group, pos ):
		ui.Sprite.__init__ ( self, cb, group )
		self.image = images.load_image ( COLORS_IMAGE, CANVAS_MASK )
		self.image = images.lower_surface ( self.image )
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.sound = sounds.load_sound ( COLORS_SOUND )

	def check_click ( self, pos ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			try:
				pos = [ pos[0] - self.rect.left, pos[1] - self.rect.top ]
				color = self.image.get_at ( pos )
				self.cb.set_color ( color )
				sounds.play_sound ( self.sound )
				return True
			except IndexError:
				return False
		return False

class cursor ( ui.Sprite ):
	def __init__ ( self, cb, group, pos ):
		ui.Sprite.__init__ ( self, cb, group )
		self.images = images.set ( images.load_image ( CURSOR_IMAGE, CANVAS_MASK ), images.load_image ( PAD_CURSOR_IMAGE, CANVAS_MASK ) )
		self.image = self.images.get ()
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.alt = False

	def change_cursor ( self ):
		if self.alt:
			self.alt = False
			self.image = self.images.set ( 0 )
		else:
			self.alt = True
			self.image = self.images.set ( 1 )

	def check_cursor ( self, pos, buttons ):
		self.set_pos ( pos )
		return True

class undo_button ( ui.Sprite ):
	def __init__ ( self, cb, group, pos ):
		ui.Sprite.__init__ ( self, cb, group )
		self.highlighted = False
		self.images = images.set ( images.load_image ( UNDO_IMAGE, CANVAS_MASK ), images.load_image ( UNDO_HIGHLIGHT_IMAGE, CANVAS_MASK ) )
		self.image = self.images.get ()
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.sound = sounds.load_sound ( UNDO_SOUND )

	def check_cursor ( self, pos, buttons ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			self.highlighted = True
			self.image = self.images.set ( 1 )
			return True
		if self.highlighted:
			self.image = self.images.set ( 0 )
			self.highlighted = False
		return False

	def check_click ( self, pos ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			sounds.play_sound ( self.sound )
			self.cb.undo ()
			return True
		return False

class snapshot_button ( ui.Sprite ):
	def __init__ ( self, cb, group, pos ):
		ui.Sprite.__init__ ( self, cb, group )
		self.highlighted = False
		self.images = images.set ( images.load_image ( SNAPSHOT_IMAGE, CANVAS_MASK ), images.load_image ( SNAPSHOT_HIGHLIGHT_IMAGE, CANVAS_MASK ) )
		self.image = self.images.get ()
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.sound = sounds.load_sound ( SNAPSHOT_SOUND )

	def check_cursor ( self, pos, buttons ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			self.highlighted = True
			self.image = self.images.set ( 1 )
			return True
		if self.highlighted:
			self.image = self.images.set ( 0 )
			self.highlighted = False
		return False

	def check_click ( self, pos ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			sounds.play_sound ( self.sound )
			self.cb.snapshot ()
			return True
		return False

class tool_button ( ui.Sprite ):
	def __init__ ( self, cb, group, pos ):
		ui.Sprite.__init__ ( self, cb, group )
		self.highlighted = False
		self.images = images.set ( images.load_image ( TOOL_IMAGE, CANVAS_MASK ), images.load_image ( TOOL_HIGHLIGHT_IMAGE, CANVAS_MASK ) )
		self.image = self.images.get ()
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.sound = sounds.load_sound ( TOOL_SOUND )

	def check_cursor ( self, pos, buttons ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			self.highlighted = True
			self.image = self.images.set ( 1 )
			return True
		if self.highlighted:
			self.image = self.images.set ( 0 )
			self.highlighted = False
		return False

	def check_click ( self, pos ):
		if self.rect.collidepoint ( pos[0], pos[1] ):
			sounds.play_sound ( self.sound )
			self.cb.tool ()
			return True
		return False

class left_button ( ui.Sprite ):
	def __init__ ( self, cb, group, pos ):
		ui.Sprite.__init__ ( self, cb, group )
		self.highlighted = False
		image1 = pygame.Surface ( [ 10, 18 ] )
		image1.blit ( images.load_image ( ARROWS_IMAGE ), ( 0, 0 ), ( 0, 0, 10, 18 ) )
		image1.set_colorkey ( CANVAS_MASK )
		image2 = pygame.Surface ( [ 10, 18 ] )
		image2.blit ( images.load_image ( ARROWS_HIGHLIGHT_IMAGE ), ( 0, 0 ), ( 0, 0, 10, 18 ) )
		image2.set_colorkey ( CANVAS_MASK )
		self.images = images.set ( image1, image2 )
		self.image = self.images.get ()
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.sound = sounds.load_sound ( ARROWS_SOUND )

	def check_cursor ( self, pos, buttons ):
		if self.rect.left <= pos[0] and self.rect.right >= pos[0]:
			self.highlighted = True
			self.image = self.images.set ( 1 )
			return True
		if self.highlighted:
			self.image = self.images.set ( 0 )
			self.highlighted = False
		return False

	def check_click ( self, pos ):
		if self.rect.left <= pos[0] and self.rect.right >= pos[0]:
			sounds.play_sound ( self.sound )
			self.cb.prev_canvas ()
			return True
		return False

class right_button ( ui.Sprite ):
	def __init__ ( self, cb, group, pos ):
		ui.Sprite.__init__ ( self, cb, group )
		self.highlighted = False
		image1 = pygame.Surface ( [ 10, 18 ] )
		image1.blit ( images.load_image ( ARROWS_IMAGE ), ( 0, 0 ), ( 13, 0, 10, 18 ) )
		image1.set_colorkey ( CANVAS_MASK )
		image2 = pygame.Surface ( [ 10, 18 ] )
		image2.blit ( images.load_image ( ARROWS_HIGHLIGHT_IMAGE ), ( 0, 0 ), ( 13, 0, 10, 18 ) )
		image2.set_colorkey ( CANVAS_MASK )
		self.images = images.set ( image1, image2 )
		self.image = self.images.get ()
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.sound = sounds.load_sound ( ARROWS_SOUND )

	def check_cursor ( self, pos, buttons ):
		if self.rect.left <= pos[0] and self.rect.right >= pos[0]:
			self.highlighted = True
			self.image = self.images.set ( 1 )
			return True
		if self.highlighted:
			self.image = self.images.set ( 0 )
			self.highlighted = False
		return False

	def check_click ( self, pos ):
		if self.rect.left <= pos[0] and self.rect.right >= pos[0]:
			sounds.play_sound ( self.sound )
			self.cb.next_canvas ()
			return True
		return False

class selected_color ( ui.Sprite ):
	def __init__ ( self, cb, group, pos, color ):
		ui.Sprite.__init__ ( self, cb, group )
		self.image = pygame.Surface ( ( 30, 30 ) ).convert ()
		self.rect = self.image.get_rect ()
		self.set_pos ( pos )
		self.set_color ( color )

	def set_color ( self, color ):
		self.color = color
		self.image.fill ( self.color )
		self.image = images.lower_surface ( self.image )

class chrome ( ui.Sprite ):
	def __init__ ( self, cb, group ):
		ui.Sprite.__init__ ( self, cb, group )
		self.image = pygame.Surface ( DISPLAY_SIZE ).convert ()
		self.image.fill ( CHROME_COLOR )
		self.image = images.raise_surface ( self.image )
		self.rect = self.image.get_rect ()

class message ( ui.Sprite ):
	def __init__ ( self, cb, group, message ):
		ui.Sprite.__init__ ( self, cb, group )
		self.image = text.textOutline ( DEFAULT_FONT, message, FONT_COLOR, OUTLINE_COLOR )
		self.rect = self.image.get_rect ()
		self.alpha = 255
		self.start_pos ()

	def start_pos ( self ):
		w = ( self.cb.Canvas.rect.width - self.rect.width ) / 2 + 20
		h = ( self.cb.Canvas.rect.height - self.rect.height ) / 2 + 20
		self.set_pos ( [ w, h ] )

	def shift ( self ):
		self.alpha = self.alpha - 5
		if self.alpha < 1:
			self.kill ()
		else:
			self.image.set_alpha ( self.alpha )

class pad ( ui.Sprite ):
	def __init__ ( self, cb, group, pos ):
		ui.Sprite.__init__ ( self, cb, group )
		self.image = pygame.Surface ( CANVAS_SIZE ).convert ()
		self.rect = self.image.get_rect ()
		self.reset ()
		self.set_pos ( pos )
		self.point = None
		self.sound = sounds.load_sound ( DRAWING_SOUND )

	def reset ( self ):
		self.image.fill ( CANVAS_MASK )
		self.image.set_colorkey ( CANVAS_MASK )
		self.image = images.lower_surface ( self.image )

	def check_cursor ( self, pos, buttons ):
		if not ( buttons[0] or buttons[2] ):
			if self.point:
				self.point = None
				self.image = images.lower_surface ( self.image )
		elif self.cb.doodle:
			if buttons[0]:
				pos = [ pos[0] - self.rect.left, pos[1] - self.rect.top ]
				if self.point:
					sounds.play_sound ( self.sound )
					pygame.draw.line ( self.image, self.cb.Selected_color.color, self.point, pos, 10 )
				self.point = pos
			elif buttons[2]:
				pos = [ pos[0] - self.rect.left, pos[1] - self.rect.top ]
				if self.point:
					sounds.play_sound ( self.sound )
					pygame.draw.line ( self.image, CANVAS_MASK, self.point, pos, 10 )
				self.point = pos
